"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.create = void 0;
var nanoevents_1 = require("nanoevents");
var optimisticUi_1 = require("./optimisticUi");
var sync_1 = require("./sync");
var globalWindow = typeof window === "undefined" ? {} : window;
function create(params) {
    var initialData = params.initialData, network = params.network;
    var onError = params.onError ||
        (function (err) {
            throw err;
        });
    var window = params.window || globalWindow;
    var debugConfig = {
        failingWrites: false,
        pretendOffline: false,
        optimisticUIEnabled: true,
    };
    var canonData = initialData;
    var events = nanoevents_1.createNanoEvents();
    var _a = sync_1.sync({
        canonData: canonData,
        network: network,
        onChange: function (kind, id) { return events.emit("change", [kind, id]); },
        onConnectivityChange: function () { return events.emit("connectivity-changed", undefined); },
        onError: onError,
        shouldCrashWrites: function () { return debugConfig.failingWrites; },
    }), connectivity = _a.connectivity, observe = _a.observe, push = _a.push;
    var _b = optimisticUi_1.optimisticUi({
        commitTransaction: push,
        onChangePendingTransactionCount: function () {
            return events.emit("pending-transaction-count-changed", undefined);
        },
        onError: onError,
        onRerenderKind: function (kind, id) { return events.emit("change", [kind, id]); },
        optimisticUIEnabled: function () { return debugConfig.optimisticUIEnabled; },
    }), addTransaction = _b.addTransaction, pendingTransactionCount = _b.pendingTransactionCount, withPendingTransactions = _b.withPendingTransactions;
    window.addEventListener("beforeunload", function (e) {
        // We can't ever save lost data anymore
        if (connectivity() === "crashed")
            return;
        if (pendingTransactionCount() === 0)
            return;
        e.preventDefault();
        e.returnValue = "";
    });
    var db = {
        _internals: {
            canonData: canonData,
            debugConfig: debugConfig,
            events: events,
            pendingTransactionCount: pendingTransactionCount,
            withPendingTransactions: withPendingTransactions,
        },
        create: function (kind, id, value) {
            addTransaction({ kind: kind, id: id, creation: value });
        },
        connectivity: function () {
            return debugConfig.pretendOffline ? "offline" : connectivity();
        },
        observe: observe,
        update: function (kind, id, delta) {
            addTransaction({ kind: kind, id: id, delta: delta });
        },
    };
    return db;
}
exports.create = create;
//# sourceMappingURL=create.js.map