"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.optimisticUi = void 0;
function optimisticUi(params) {
    var commitTransaction = params.commitTransaction, onChangePendingTransactionCount = params.onChangePendingTransactionCount, onError = params.onError, onRerenderKind = params.onRerenderKind, optimisticUIEnabled = params.optimisticUIEnabled;
    var pendingTransactions = [];
    return {
        addTransaction: function (transaction) {
            var kind = transaction.kind, id = transaction.id;
            pendingTransactions.push(transaction);
            onChangePendingTransactionCount();
            // Rerender immediately to show optimistic update
            onRerenderKind(kind, id);
            commitTransaction(transaction)
                .then(function () {
                pendingTransactions = pendingTransactions.filter(function (t) { return t !== transaction; });
                onChangePendingTransactionCount();
                // We have removed the pending transaction asynchronously,
                // and the server may have already added changes on top,
                // which the UI does not show yet.
                onRerenderKind(kind, id);
            })
                .catch(onError);
        },
        pendingTransactionCount: function () { return pendingTransactions.length; },
        withPendingTransactions: function (data) {
            if (!optimisticUIEnabled())
                return data;
            var result = {};
            Object.keys(data).forEach(function (k) {
                result[k] = __assign({}, data[k]);
            });
            // Apply pending transactions on top of the canon version
            pendingTransactions.forEach(function (transaction) {
                if ("delta" in transaction) {
                    var prev = result[transaction.kind][transaction.id];
                    // If we have not fetched the item yet,
                    // we don’t need to transform it optimistically
                    if (!prev)
                        return;
                    result[transaction.kind][transaction.id] = {
                        revision: "optimistic",
                        value: transaction.delta(prev.value),
                    };
                }
                else {
                    result[transaction.kind][transaction.id] = {
                        revision: "optimistic",
                        value: transaction.creation,
                    };
                }
            });
            return result;
        },
    };
}
exports.optimisticUi = optimisticUi;
//# sourceMappingURL=optimisticUi.js.map