"use strict";
/**
 * This over-complicated state machine ensures that we throttle
 * our transition to the offline mode, and then stay longer in it,
 * without jumping back and forth too often.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeConnectivity = void 0;
function makeConnectivity(params) {
    var onConnectivityChange = params.onConnectivityChange;
    var current = "connecting";
    var pendingOnlineState;
    var onlineDelayTimer;
    var offlineDelayTimerOn = false;
    var offlineDelayTimer;
    function setReal(c) {
        current = c;
        onConnectivityChange();
    }
    var transitions = {
        on: {
            on: function (newConnectivity) {
                clearTimeout(offlineDelayTimer);
                offlineDelayTimerOn = false;
                setReal(newConnectivity);
            },
            off: function () {
                if (!offlineDelayTimerOn) {
                    offlineDelayTimerOn = true;
                    offlineDelayTimer = setTimeout(function () {
                        setReal("offline");
                        offlineDelayTimerOn = false;
                    }, 800);
                }
            },
        },
        off: {
            off: function () {
                clearTimeout(onlineDelayTimer);
                pendingOnlineState = undefined;
            },
            on: function (newConnectivity) {
                if (pendingOnlineState) {
                    pendingOnlineState = newConnectivity;
                }
                else {
                    pendingOnlineState = newConnectivity;
                    onlineDelayTimer = setTimeout(function () {
                        if (!pendingOnlineState)
                            return;
                        setReal(pendingOnlineState);
                        pendingOnlineState = undefined;
                    }, 800);
                }
            },
        },
    };
    return {
        get: function () { return current; },
        set: function (newConnectivity) {
            if (current === "crashed")
                return;
            if (newConnectivity === "crashed")
                return setReal(newConnectivity);
            var from = current === "offline" ? "off" : "on";
            var to = newConnectivity === "offline" ? "off" : "on";
            transitions[from][to](newConnectivity);
        },
    };
}
exports.makeConnectivity = makeConnectivity;
//# sourceMappingURL=connectivity.js.map