"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.sync = void 0;
var connectivity_1 = require("./connectivity");
var push_1 = require("./push");
function makeCounters() {
    var stringify = function (a, b) { return a + "-" + b; };
    var data = {};
    return {
        add: function (a, b, f) {
            var kind = stringify(a, b);
            // Cancellation is still not in the protocol,
            // so this is stub logic for now that never cancels.
            if (!data[kind])
                data[kind] = { count: 1, cancel: f() };
            return function () { };
            // if (!data[kind] || data[kind].count === 0) {
            //   data[kind] = { count: 0, cancel: f() };
            // }
            // data[kind].count += 1;
            // return function() {
            //   data[kind].count -= 1;
            //   if (data[kind].count === 0) {
            //     data[kind].cancel();
            //     data[kind].cancel = () => {
            //       throw new Error(`Assertion error`);
            //     };
            //   }
            // };
        },
    };
}
function sync(params) {
    var canonData = params.canonData, network = params.network, onChange = params.onChange, onConnectivityChange = params.onConnectivityChange, onError = params.onError, shouldCrashWrites = params.shouldCrashWrites;
    var connectivity = connectivity_1.makeConnectivity({
        onConnectivityChange: onConnectivityChange,
    });
    var pushesInFlight = {};
    var net = network({
        onChange: function (_a) {
            var kind = _a.kind, id = _a.id, revision = _a.revision, value = _a.value;
            var kindCollection = canonData[kind];
            kindCollection[id] = { revision: revision, value: value };
            onChange(kind, id);
        },
        onConnectivityChange: connectivity.set,
        onError: onError,
        onPushResult: function (pushId, result) {
            if (!pushesInFlight[pushId])
                return;
            pushesInFlight[pushId](result);
            delete pushesInFlight[pushId];
        },
    });
    var push = push_1.makePush({
        canonData: canonData,
        onError: function (err) {
            connectivity.set("crashed");
            throw err;
        },
        onNetPush: function (pushParams) {
            return new Promise(function (resolve) {
                pushesInFlight[pushParams.pushId] = resolve;
                net.push(pushParams);
            });
        },
        onUpdate: function (_a) {
            var kind = _a.kind, id = _a.id, newValue = _a.newValue, newRevision = _a.newRevision;
            var kindCollection = canonData[kind];
            kindCollection[id] = {
                revision: newRevision,
                value: newValue,
            };
        },
        shouldCrashWrites: shouldCrashWrites,
    });
    var counters = makeCounters();
    return {
        connectivity: connectivity.get,
        observe: function (kind, id) {
            return counters.add(kind, id, function () {
                return net.getAndObserve(kind, id);
            });
        },
        push: push,
    };
}
exports.sync = sync;
//# sourceMappingURL=index.js.map