"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !exports.hasOwnProperty(p)) __createBinding(exports, m, p);
}
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeWsProtocolAdapter = void 0;
var reconnecting_websocket_1 = __importDefault(require("reconnecting-websocket"));
var serialization_1 = require("./serialization");
__exportStar(require("./memoryServer"), exports);
var server_1 = require("./server");
Object.defineProperty(exports, "runWebsocketServer", { enumerable: true, get: function () { return server_1.runWebsocketServer; } });
function makeWsProtocolAdapter(params) {
    var url = params.url, _socket = params._socket;
    var serialization = params.serialization || serialization_1.jsonSerialization;
    var socket = _socket || new reconnecting_websocket_1.default(url);
    var pingTimer;
    var timeoutTimer;
    function restartPingMachine() {
        clearTimeout(pingTimer);
        clearTimeout(timeoutTimer);
        pingTimer = setTimeout(function () {
            send({ action: "ping" });
            timeoutTimer = setTimeout(function () { return socket.reconnect(); }, 15000);
        }, 15000);
    }
    restartPingMachine();
    var send = function (input) {
        socket.send(JSON.stringify(input));
    };
    var messagesOnEveryReconnect = [];
    return {
        adapter: function (_a) {
            var onChange = _a.onChange, onConnectivityChange = _a.onConnectivityChange, onError = _a.onError, onPushResult = _a.onPushResult;
            socket.onopen = function () {
                onConnectivityChange("online");
                restartPingMachine();
                messagesOnEveryReconnect.forEach(send);
            };
            socket.onclose = function () { return onConnectivityChange("offline"); };
            socket.onerror = function () {
                return onError(new Error("WebSocket error. We can’t retrieve details about the error because the browser does not provide them for security reasons."));
            };
            var handlers = {
                clientError: function (msg) {
                    onError(new Error(msg.message));
                },
                pong: function () { },
                pushResult: function (_a) {
                    var id = _a.id, kind = _a.kind, newRevision = _a.newRevision, newValue = _a.newValue, pushId = _a.pushId, result = _a.result;
                    if (result === "success")
                        onPushResult(pushId, {
                            newRevision: newRevision,
                            newValue: serialization.decode(newValue, { id: id, kind: kind }),
                        });
                    else
                        onPushResult(pushId, result);
                },
                update: function (_a) {
                    var id = _a.id, kind = _a.kind, revision = _a.revision, value = _a.value;
                    return onChange({
                        kind: kind,
                        id: id,
                        revision: revision,
                        value: serialization.decode(value, { id: id, kind: kind }),
                    });
                },
            };
            socket.onmessage = function (event) {
                restartPingMachine();
                var msg = JSON.parse(event.data);
                var action = msg.action;
                if (!action)
                    return console.warn("Unrecognized message", msg);
                var handler = handlers[action];
                if (!handler)
                    return console.warn("Unrecognized message", msg);
                handler(msg);
            };
            return {
                getAndObserve: function (kind, id) {
                    var msg = { action: "subscribe", kind: kind, id: id };
                    if (socket.readyState === 1)
                        send(msg);
                    messagesOnEveryReconnect.push(msg);
                    return function () { };
                },
                push: function (_a) {
                    var kind = _a.kind, id = _a.id, pushId = _a.pushId, lastSeenRevision = _a.lastSeenRevision, value = _a.value;
                    send({
                        action: "push",
                        kind: kind,
                        id: id,
                        pushId: pushId,
                        lastSeenRevision: lastSeenRevision,
                        value: serialization.encode(value, { id: id, kind: String(kind) }),
                    });
                },
            };
        },
        auth: function (token) {
            var msg = { action: "auth", token: token };
            var currentIndex = messagesOnEveryReconnect.findIndex(function (m) { return m.action === "auth"; });
            if (currentIndex === -1)
                messagesOnEveryReconnect.push(msg);
            else
                messagesOnEveryReconnect[currentIndex] = msg;
            if (socket.readyState === 1)
                send(msg);
        },
    };
}
exports.makeWsProtocolAdapter = makeWsProtocolAdapter;
//# sourceMappingURL=index.js.map