"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.runWebsocketServer = void 0;
var nanoevents_1 = require("nanoevents");
var ws_1 = __importDefault(require("ws"));
var serialization_1 = require("../serialization");
var authQueue_1 = require("./authQueue");
var types_1 = require("./types");
function runWebsocketServer(params) {
    var auth = params.auth, onChangeData = params.onChangeData, onRequestData = params.onRequestData;
    var serialization = params.serialization || serialization_1.jsonSerialization;
    var events = nanoevents_1.createNanoEvents();
    var latestCopies = {};
    events.on("change", function (kv) {
        latestCopies[types_1.stringy(kv)] = kv.value;
    });
    var listeningTo = [];
    function getAndObserve(key, close, onData) {
        var cancel = events.on("change", function (eventKV) {
            if (types_1.stringy(key) === types_1.stringy(eventKV)) {
                onData(eventKV.value);
            }
        });
        if (listeningTo.includes(types_1.stringy(key))) {
            var last = latestCopies[types_1.stringy(key)];
            if (last)
                onData(last);
            return cancel;
        }
        listeningTo.push(types_1.stringy(key));
        onRequestData(__assign(__assign({}, key), { close: close, send: function (value) { return events.emit("change", __assign(__assign({}, key), { value: value })); } }));
        return cancel;
    }
    var server = params._ws ||
        new ws_1.default.Server({
            port: params.port,
            server: params.server,
        });
    function heartbeat() {
        this.lifeFlag = "good";
    }
    setInterval(function ping() {
        server.clients.forEach(function (socket) {
            if (socket.lifeFlag === undefined) {
                socket.lifeFlag = "good";
                socket.on("pong", heartbeat);
                socket.on("message", heartbeat);
                return;
            }
            if (socket.lifeFlag === "good") {
                socket.lifeFlag = "sleepy";
                socket.ping();
                return;
            }
            if (socket.lifeFlag === "sleepy") {
                socket.terminate();
                return;
            }
        });
    }, 60000);
    server.on("connection", function (socket) {
        var _this = this;
        var authQueue = authQueue_1.makeAuthQueue({
            onTerminate: function () { return socket.terminate(); },
        });
        var latestToken;
        var evaluateToken = function () {
            if (!latestToken)
                return;
            if (!auth) {
                console.log("Ignoring auth message");
                return;
            }
            authQueue.newAuthIncoming(auth
                .parseToken(latestToken, { close: function () { return socket.terminate(); } })
                .catch(function () {
                socket.terminate();
                return undefined;
            }));
        };
        var reEvaluateToken = setInterval(evaluateToken, 5 * 60000);
        socket.on("close", function () { return clearInterval(reEvaluateToken); });
        var send = function (obj) {
            if (socket.readyState !== 1)
                return;
            socket.send(JSON.stringify(obj));
        };
        var handlers = {
            ping: function () { return send({ action: "pong" }); },
            auth: function (msg) {
                latestToken = msg.token;
                evaluateToken();
            },
            push: function (msg) { return __awaiter(_this, void 0, void 0, function () {
                var kind, id, lastSeenRevision, value, details;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            kind = msg.kind, id = msg.id, lastSeenRevision = msg.lastSeenRevision, value = msg.value;
                            if (!auth) return [3 /*break*/, 2];
                            return [4 /*yield*/, authQueue.details()];
                        case 1:
                            details = _a.sent();
                            if (!auth.canWrite({ auth: details, kind: kind, id: id })) {
                                send({
                                    action: "clientError",
                                    message: "Tried to write " + kind + " " + id + " without permission",
                                });
                                return [2 /*return*/];
                            }
                            _a.label = 2;
                        case 2:
                            onChangeData({
                                kind: kind,
                                lastSeenRevision: lastSeenRevision,
                                id: id,
                                value: serialization.decode(value, { kind: kind, id: id }),
                                close: function () { return socket.terminate(); },
                            })
                                .then(function (result) {
                                if (result === "conflict")
                                    send({
                                        action: "pushResult",
                                        pushId: msg.pushId,
                                        result: result,
                                    });
                                else {
                                    send({
                                        action: "pushResult",
                                        id: id,
                                        kind: kind,
                                        pushId: msg.pushId,
                                        result: "success",
                                        newRevision: result.newRevision,
                                        newValue: serialization.encode(result.newValue, { id: id, kind: kind }),
                                    });
                                    events.emit("change", {
                                        kind: kind,
                                        id: id,
                                        value: { revision: result.newRevision, value: result.newValue },
                                    });
                                }
                            })
                                .catch(function (err) {
                                send({
                                    action: "pushResult",
                                    pushId: msg.pushId,
                                    result: "internalError",
                                });
                                throw err;
                            });
                            return [2 /*return*/];
                    }
                });
            }); },
            subscribe: function (msg) {
                var id = msg.id, kind = msg.kind;
                var cancel = getAndObserve({ kind: kind, id: id }, function () { return socket.terminate(); }, function (v) { return __awaiter(_this, void 0, void 0, function () {
                    var details;
                    return __generator(this, function (_a) {
                        switch (_a.label) {
                            case 0:
                                if (!auth) return [3 /*break*/, 2];
                                return [4 /*yield*/, authQueue.details()];
                            case 1:
                                details = _a.sent();
                                if (!auth.canRead({ auth: details, kind: kind, id: id })) {
                                    send({
                                        action: "clientError",
                                        message: "Tried to read " + kind + " " + id + " without permission",
                                    });
                                    return [2 /*return*/];
                                }
                                _a.label = 2;
                            case 2:
                                send({
                                    action: "update",
                                    id: id,
                                    kind: kind,
                                    revision: v.revision,
                                    value: serialization.encode(v.value, { id: id, kind: kind }),
                                });
                                return [2 /*return*/];
                        }
                    });
                }); });
                socket.on("close", cancel);
            },
        };
        var handleMsg = function (msg) {
            var action = msg.action;
            var handler = handlers[action];
            if (!handler)
                return console.warn("Unrecognized message", msg);
            handler(msg);
        };
        socket.on("message", function incoming(data) {
            function attempt(f) {
                try {
                    return f();
                }
                catch (err) {
                    return undefined;
                }
            }
            var msg = attempt(function () { return JSON.parse(data.toString()); });
            if (!msg) {
                console.warn("Received invalid message format");
                socket.terminate();
                return;
            }
            handleMsg(msg);
        });
    });
}
exports.runWebsocketServer = runWebsocketServer;
//# sourceMappingURL=index.js.map