"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeWsMock = void 0;
function makeWsMock() {
    var connectionListener;
    var server = {
        clients: new Set(),
        on: function (event, cb) {
            if (event === "connection") {
                if (connectionListener)
                    throw new Error("Not implemented");
                connectionListener = cb;
                return server;
            }
            throw new Error("Event " + event + " not implemented");
        },
    };
    return {
        client: function (onServerMessage) {
            var onClientMessage;
            var socket = {
                on: function (event, cb) {
                    if (event === "message") {
                        if (onClientMessage)
                            throw new Error("Not implemented");
                        onClientMessage = cb;
                        return;
                    }
                    if (event === "close")
                        return;
                    throw new Error("event " + event + " not implemented");
                },
                readyState: 1,
                send: function (data) { return onServerMessage(JSON.parse(data)); },
            };
            if (!connectionListener)
                throw new Error("Not implemented");
            connectionListener(socket);
            return {
                send: function (msg) {
                    return onClientMessage(Buffer.from(JSON.stringify(msg)));
                },
            };
        },
        server: server,
    };
}
exports.makeWsMock = makeWsMock;
//# sourceMappingURL=ws-mock.js.map