"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeHooks = void 0;
function flat(list) {
    return list.reduce(function (a, b) { return a.concat(b); });
}
function makeMassUpdater(db) {
    var updater = function (_options, kind, id, delta) {
        db.update(kind, id, delta);
    };
    return function (a, b, c, d) {
        return d ? updater(a, b, c, d) : updater({}, a, b, c);
    };
}
function makeUpdater(massUpdater, kind, id) {
    var updater = function (options, delta) {
        massUpdater(options, kind, id, delta);
    };
    return (function (a, b) {
        if (typeof a === "string")
            return updater({}, function (prev) {
                var _a;
                return (__assign(__assign({}, prev), (_a = {}, _a[a] = b, _a)));
            });
        updater(b ? a : {}, b || a);
    });
}
function makeHooks(params) {
    var React = params.React, useTryDB = params.useTryDB;
    function useRerender() {
        var _a = React.useReducer(function (x) { return x + 1; }, 0), rerender = _a[1];
        return function () {
            rerender();
        };
    }
    var useDataMap = function (definition) {
        var db = useTryDB();
        if (db === "missing-provider")
            throw new Error("The component you render needs a DBProvider grand-parent");
        var items = flat(Object.keys(definition).map(function (kind) {
            return definition[kind].map(function (id) { return ({ kind: kind, id: id }); });
        }));
        var definitionHash = JSON.stringify(definition);
        var rerender = useRerender();
        React.useEffect(function () {
            return db._internals.events.on("change", function (k) {
                if (items.some(function (_a) {
                    var kind = _a.kind, id = _a.id;
                    return kind === k[0] && id === k[1];
                }))
                    rerender();
            });
        }, [definitionHash]);
        React.useEffect(function () {
            var cancels = items.map(function (_a) {
                var kind = _a.kind, id = _a.id;
                return db.observe(kind, id);
            });
            return function () { return cancels.forEach(function (f) { return f(); }); };
        }, [definitionHash]);
        var allData = db._internals.withPendingTransactions(db._internals.canonData);
        var isLoaded = items.every(function (_a) {
            var kind = _a.kind, id = _a.id;
            return Boolean(allData[kind][id]);
        });
        var data = {};
        if (isLoaded) {
            items.forEach(function (_a) {
                var kind = _a.kind, id = _a.id;
                data[kind] = data[kind] || {};
                var atom = allData[kind][id];
                if (!atom)
                    throw new Error("Oncilla internal consistency error, unexpectedly missing " + kind + "-" + id + " in the data");
                var kindCollection = data[kind];
                kindCollection[id] = atom.value;
            });
        }
        return [
            isLoaded ? data : "loading",
            React.useCallback(makeMassUpdater(db), [db]),
        ];
    };
    return {
        useConnectivity: function () {
            var db = useTryDB();
            var rerender = useRerender();
            React.useEffect(function () {
                return db === "missing-provider"
                    ? undefined
                    : db._internals.events.on("connectivity-changed", rerender);
            }, [db, rerender]);
            if (db === "missing-provider")
                return "online";
            return db.connectivity();
        },
        useCreate: function (kind) {
            var db = useTryDB();
            if (db === "missing-provider")
                throw new Error("The component you render needs a DBProvider grand-parent");
            return function (id, value) {
                db.create(kind, id, value);
            };
        },
        useData: function (kind, id) {
            var _a;
            var _b = useDataMap((_a = {},
                _a[kind] = [id],
                _a)), data = _b[0], update = _b[1];
            return [
                data === "loading" ? "loading" : data[kind][id],
                makeUpdater(update, kind, id),
            ];
        },
        useMultipleData: function useMultipleData(kind, ids) {
            var _a;
            var definition = (_a = {},
                _a[kind] = ids,
                _a);
            var _b = useDataMap(definition), data = _b[0], update = _b[1];
            return [
                data === "loading" ? "loading" : data[kind],
                function (id, delta) {
                    return update(kind, id, delta);
                },
            ];
        },
        useOncillaDebug: function () {
            var db = useTryDB();
            if (db === "missing-provider")
                throw new Error("The component you render needs a DBProvider grand-parent");
            var _a = db._internals, debugConfig = _a.debugConfig, events = _a.events;
            var rerender = useRerender();
            React.useEffect(function () { return events.on("debug-config-changed", rerender); }, []);
            return [
                debugConfig,
                React.useCallback(function (updates) {
                    Object.assign(debugConfig, updates);
                    events.emit("debug-config-changed", undefined);
                }, []),
            ];
        },
        usePendingChanges: function () {
            var db = useTryDB();
            var rerender = useRerender();
            React.useEffect(function () {
                return db === "missing-provider"
                    ? undefined
                    : db._internals.events.on("pending-transaction-count-changed", rerender);
            }, []);
            if (db === "missing-provider")
                return 0;
            return db._internals.pendingTransactionCount();
        },
    };
}
exports.makeHooks = makeHooks;
//# sourceMappingURL=hooks.js.map